//
//  STTimeline.h
//  Kahuna
//
//  Created by Davyd Geyl on 31/07/09.
//  Copyright 2009 Sportstec Ltd. All rights reserved.
//

@import SCNFoundation;

#import "STTimelineInstance.h"

extern NSString* const STTimelineDidAddEventsNotification;
extern NSString* const STTimelineDidRemoveEventsNotification;
extern NSString* const STTimelineSortingDidChangeNotification;

extern NSString* const STTimelineDidChangeEventNameNotification;
extern NSString* const STTimelineDidChangeEventNameTextAttributesNotification;

extern NSString* const STTimelineInstancesKey; 

extern NSString* const STTimelineDidAddInstancesNotification;
extern NSString* const STTimelineDidRemoveInstancesNotification;
extern NSString* const STTimelineDidAlterInstancesNotification;

extern NSString* const STTimelineKey;
extern NSString* const STTimelineEventsKey;
extern NSString* const STTimelineEventKey;
extern NSString* const STTimelineQuickLookImageDataKey;
extern NSString* const STTimelineQuickLookScaledImageDataKey;

extern NSString *const STTimelineInstanceDidAddTagNotification;
extern NSString *const STTimelineInstanceDidRemoveTagNotification;
extern NSString *const STTimelineInstanceKey;
extern NSString *const STTimelineInstanceTagKey;

extern CGFloat const STTimelineScaledQuickLookImageWidth;

typedef NS_ENUM(NSInteger, STInstancesArrangingStyle)
{
    STInstancesMergedStyle = 0,
    STInstancesSequentialStyle = 1,
    STInstancesSelectionOrderStyle = 3
};

typedef NS_ENUM(NSInteger, STImportTimelineOption)
{
	STImportTimelineOptionReplace = 0,
    STImportTimelineOptionAdd = 1,
    STImportTimelineOptionMerge = 2
};

@class STTimelineEvent;
@class STTimelineInstance;
@class STTimelineInstanceTag;
@class STSortDescriptor;

@protocol STTimelineDelegate;

@interface STTimeline : STSubDataModel <NSCoding, STSortMasterInterface, STHolderInterface>
{
    NSMutableArray* _events;
}

- (instancetype)initWithEvents:(NSArray*)events;

@property (nonatomic, weak) id<STTimelineDelegate> delegate;
@property (nonatomic, assign) NSTimeInterval startCodingTimeInterval;
@property (nonatomic, readonly) NSArray* timelineInstanceTags;
@property (nonatomic, readonly) NSArray* events;
@property (nonatomic, readonly) NSUInteger numOfAllInstances;

@property (nonatomic, assign, getter = isEditable) BOOL editable;

@property (nonatomic, readonly) NSTimeInterval contentDuration;
@property (nonatomic, readonly) NSTimeInterval currentTime;
@property (nonatomic, readonly) BOOL isThereFuture;

#pragma mark - Update Events
/**
 Adds an array of events to the timeline.
 Does NOT notify the UI to update

 @param events array of events to add
 */
- (void)addEvents:(NSArray*)events;

/**
 Adds an event to the timeline
 Does NOT notify the UI to update

 @param event event to add
 */
- (void)addEvent:(STTimelineEvent*)event;

/**
 Removes an array of events from the timeline
 Does NOT notify the UI to update

 @param event event to remove
 */
- (void)removeEvent:(STTimelineEvent*)event;

/**
 Remove an array of events from the timeline
 Does NOT notify the UI to update

 @param events an array of events to remove
 */
- (void)removeEvents:(NSArray*)events;

/**
 Removes all events from the timeline
 Does NOT notify the UI to update
 */
- (void)removeAllEvents;

#pragma mark - Update Events and Notify UI
/**
 Adds an `STTimelineEvent to the timeline and notifies the UI to update

 @param event an event to add
 */
- (void)addEventAndNotify:(STTimelineEvent *)event;
/**
 Adds an array of events to the timeline and notifies the UI to update.

 @param events an array of events to add
 */
- (void)addEventsAndNotify:(NSArray *)events;

/**
 Removes an event from the timeline and notifies the UI to update

 @param event an event to remove
 */
- (void)removeEventAndNotify:(STTimelineEvent *)event;

/**
 Removes an array of events from the timeline and notifies the UI to update

 @param events an array of events to remove
 */
- (void)removeEventsAndNotify:(NSArray *)events;


#pragma mark - Update Events (Undoable)
/**
 Adds an event to the timeline and notifies the UI to update.
 Adds an undo action to the timeline's undo manager.

 @param event an event to add
 */
- (void)addUserEvent:(STTimelineEvent*)event;
/**
 Adds an array of events to the timline and notifies the UI to update.
 Adds an undo action to the timeline's undo manager.

 @param events an array of events to add
 */
- (void)addUserEvents:(NSArray*)events;

/**
 Removes an event from the timeline and notifies the UI to update.
 Adds an undo action to the timeline's undo manager.

 @param event an event to remove
 */
- (void)removeUserEvent:(STTimelineEvent*)event;

/**
 Removes an array of events from the timeline and notifies the UI to update.
 Adds an undo action to the timeline's undo manager.

 @param events an array of events to remove
 */
- (void)removeUserEvents:(NSArray*)events;

#pragma mark -

- (void)eventDidChangeCodeName:(STTimelineEvent *)event;

- (void)event:(STTimelineEvent*)event didAddInstance:(STTimelineInstance*)instance;
- (void)event:(STTimelineEvent*)event didRemoveInstance:(STTimelineInstance*)instance;
- (void)event:(STTimelineEvent*)event didAlterInstance:(STTimelineInstance*)instance;

- (void)instance:(STTimelineInstance*)instance didAddTag:(STTimelineInstanceTag*)tag;
- (void)instance:(STTimelineInstance*)instance didRemoveTag:(STTimelineInstanceTag*)tag;

- (void)userCloseAllOpenInstances;

- (BOOL)canAddBlankEvent;
- (void)addUserBlankEvent;

// Sorting

@property (nonatomic, readonly) NSArray* sortDescriptors;
+ (NSArray*)availableSortDescriptors;
+ (NSInteger)indexOfAvailableDescriptorWithName:(NSString*)name;
- (void)addUserSortDescriptor:(STSortDescriptor*)sortDescriptor;
- (void)insertUserSortDescriptor:(STSortDescriptor*)sortDescriptor atIndex:(NSInteger)index;
- (void)removeUserSortDescriptor:(STSortDescriptor*)sortDescriptor;
- (void)replaceUserDescriptorAtIndex:(NSUInteger)index withDescriptor:(STSortDescriptor*)sortDescriptor;
- (void)setUserDirection:(BOOL)ascending forSortDescriptor:(STSortDescriptor*)sortDescriptor;

// changes custom sorting so the given events are placed after the event provided in the second parameter
// if the second parameter is nil, the events are put in the beginning of the events list
- (void)placeUserEvents:(NSArray*)events afterEvent:(STTimelineEvent*)event;

@end

@protocol STTimelineDelegate <STTimelineInstanceCurrentEndTimeDelegate>

- (NSTimeInterval)timelineContentDuration;
- (NSTimeInterval)timelineCurrentTime;
- (BOOL)isThereFuture;

@end
