//
//  STTimelineEvent.h
//  Kahuna
//
//  Created by Davyd Geyl on 6/08/09.
//  Copyright 2009 Sportstec Ltd. All rights reserved.
//

#import <Cocoa/Cocoa.h>
@import SCNFoundation.STProtoObject;
@import SCNFoundation.STSorting;


extern NSString* const STCodeEventCaptionAttributesDidChangeNotification;

extern NSString* const STTimelineEventDidAddInstancesNotification;
extern NSString* const STTimelineEventDidRemoveInstancesNotification;
extern NSString* const STTimelineEventDidAlterInstancesNotification;
extern NSString* const STTimelineEventInstancesKey;

@class STTimelineInstance;
@class STColor;
@class STTextObject;
@class STTimeline;
@class STTimelineInstanceTag;
 
@interface STTimelineEvent : STProtoObject <STFillObjectInterface, STCodeNameInterface, NSCoding, STSortableItemInterface, STHolderInterface, STHeldObjectInterface>
{
	NSMutableArray* _instances;
	STFillType _fillType;
	STImageFillType _imageFillType;
}

+ (instancetype)event;

@property (nonatomic, readonly) NSUInteger storedHash DEPRECATED_ATTRIBUTE;
// TODO: check the usage of uniqueID and replace it with the identifier property

@property (nonatomic, readonly) NSArray<STTimelineInstance *> *instances;

@property (nonatomic, weak) STTimeline* timeline;

@property (nonatomic, strong) STTextObject* codeNameTextObject;
@property (nonatomic, strong) NSString* codeName;
@property (nonatomic, readonly) NSString* uniqueId;
@property (nonatomic, readonly) NSString *identifier;

@property (nonatomic, strong) STColor* fillColor;

@property (nonatomic, assign) NSUInteger orderIndex;
@property (nonatomic, readonly) BOOL isOpen;
@property (nonatomic, readonly, getter = isEditable) BOOL editable;

@property (nonatomic, readonly) NSInteger numOfInstance;

@property (nonatomic, readonly) NSString *JSONStringRepresentation;
- (instancetype)initWithJSONRepresentation:(NSString *)JSONRepresentation;
- (instancetype)initWithTimelineInstances:(NSArray *)instances;

- (BOOL)isEqual:(id)object;
- (NSUInteger)hash;

#pragma mark - Update Instances
/**
 Adds an instance to the event. Does NOT notify UI.

 @param instance instance to add
 */
- (void)addInstance:(STTimelineInstance*)instance;
/**
 Removes an instance from the event. Does NOT notify UI.

 @param instance instance to remove
 */
- (void)removeInstance:(STTimelineInstance*)instance;

#pragma mark - Update Instances & Notify UI
/**
 Adds an instance to the event and notifies the UI.

 @param instance instance to add
 */
- (void)addInstanceAndNotify:(STTimelineInstance *)instance;
/**
 Adds an array of instances to the event and notifies the UI.

 @param instances array of instances to add
 */
- (void)addInstancesAndNotify:(NSArray *)instances;
/**
 Remove an instance from the event and notify the UI

 @param instance instance to remove
 */
- (void)removeInstanceAndNotify:(STTimelineInstance *)instance;
/**
 Removes an array of instances from the event and notifies the UI

 @param instances array of instances to remove
 */
- (void)removeInstancesAndNotify:(NSArray *)instances;

#pragma mark - Update Instances (Undoable)
/**
 Adds an array of instances to the event, notifies the UI, and creates an undo action on the event's undo stack

 @param instances array of instances to add
 */
- (void)addUserInstances:(NSArray*)instances;
/**
 Removes an array of instances from the event, notifies the UI, and creates an undo action on the event's undo stack

 @param instances array of instances to remove
 */
- (void)removeUserInstances:(NSArray*)instances;
/**
 Adds an instance to the event, notifies the UI, and creates an undo action on the event's undo stack

 @param instance instance to add
 */
- (void)addUserInstance:(STTimelineInstance*)instance;
/**
 Removes an array of instances from the event, notifies the UI, and creates an undo action on the event's undo stack

 @param instance instance to remove
 */
- (void)removeUserInstance:(STTimelineInstance*)instance;

#pragma mark -
// Add instance in transaction
- (void)beginAddingInstance:(STTimelineInstance*)instance;
- (void)commitAddingInstance:(STTimelineInstance*)instance;
- (void)rollbackAddingInstance:(STTimelineInstance*)instance;

// Instances from the parameter array will be processed olny if no equal instances already exists.
// New instances will be added if their time period is vacant. If a new instance overlaps one or two existing instances they will be merged.
// If two or more overrlapping instances are merged the earliest time will be the new start time, the latest end time will be the new end time; all the labels are added into the resulting instance.
- (void)mergeUserInstances:(NSArray*)instances;

// Sorting support
- (NSNumber*)numberOfInstances;

- (void)instanceDidChange:(STTimelineInstance*)instance;
- (void)instance:(STTimelineInstance*)instance didAddTag:(STTimelineInstanceTag*)tag; 
- (void)instance:(STTimelineInstance*)instance didRemoveTag:(STTimelineInstanceTag*)tag; 

// Counter support
- (NSTimeInterval)totalTimeOfInstances;
- (NSTimeInterval)timeSinceLastInstanceOpened;

- (NSInteger)numberOfPopupTags:(STTimelineInstanceTag*)timelineInstanceTag;
- (NSInteger)numberOfDistinctTagsApplied:(STTimelineInstanceTag*)timelineInstanceTag;

@end
